<?php
// $Id$

/**
 * @file
 * Hosting tests functions.
 */

/**
 * Current version of this program.
 */
define('HOSTING_TESTS_VERSION', '0.4.0');

/**
 * Minimum supported version of PHP.
 * Lifted from modules/system/system.module
 */
define('DRUPAL_MINIMUM_PHP',    '4.3.5');

/**
 * Minimum recommended value of PHP memory_limit.
 * Lifted from modules/system/system.module
 */
define('DRUPAL_MINIMUM_PHP_MEMORY_LIMIT',    '16M');

/**
 * Equivalent of t(), though currently almost a complete fake.
 * There is no language translation process, but embedded commands
 * are parsed and converted (at least that).
 *
 * Lifted from includes/install.inc
 */
function t($string, $args = array()) {
  foreach ($args as $key => $value) {
    switch ($key[0]) {
      // Escaped only
      case '@':
        // there is no check_plain() function here
        $args[$key] = htmlspecialchars($value, ENT_QUOTES);
        break;
      // Escaped and placeholder
      case '%':
      default:
        // there is no check_plain() function here
        // $args[$key] = '<em>'. check_plain($value) .'</em>';
        $args[$key] = '<em>'. htmlspecialchars($value, ENT_QUOTES) .'</em>';
        break;
      // Pass-through
      case '!':
    }
  }
  return strtr($string, $args);
}

/**
 * Parse a given byte count.
 *
 * Lifted from includes/common.inc
 *
 * @param $size
 *   A size expressed as a number of bytes with optional SI size and unit
 *   suffix (e.g. 2, 3K, 5MB, 10G).
 * @return
 *   An integer representation of the size.
 */
function parse_size($size) {
  $suffixes = array(
    '' => 1,
    'k' => 1024,
    'm' => 1048576, // 1024 * 1024
    'g' => 1073741824, // 1024 * 1024 * 1024
  );
  if (preg_match('/([0-9]+)\s*(k|m|g)?(b?(ytes?)?)/i', $size, $match)) {
    return $match[1] * $suffixes[strtolower($match[2])];
  }
}

/**
 * Perform an HTTP request.
 *
 * This is a flexible and powerful HTTP client implementation. Correctly handles
 * GET, POST, PUT or any other HTTP requests. Handles redirects.
 *
 * Lifted from includes/common.inc
 *
 * @param $url
 *   A string containing a fully qualified URI.
 * @param $headers
 *   An array containing an HTTP header => value pair.
 * @param $method
 *   A string defining the HTTP request to use.
 * @param $data
 *   A string containing data to include in the request.
 * @param $retry
 *   An integer representing how many times to retry the request in case of a
 *   redirect.
 * @return
 *   An object containing the HTTP request headers, response code, headers,
 *   data and redirect status.
 */
function drupal_http_request($url, $headers = array(), $method = 'GET', $data = NULL, $retry = 3) {

  $result = new stdClass();

  // Parse the URL and make sure we can handle the schema.
  $uri = parse_url($url);

  if ($uri == FALSE) {
    $result->error = t('Unable to parse URL @url', array('@url' => $url));
    $result->code = -1001;
    return $result;
  }

  if (!isset($uri['scheme'])) {
    $result->error = t('Missing schema in URL @url', array('@url' => $url));
    $result->code = -1002;
    return $result;
  }

  switch ($uri['scheme']) {
    case 'http':
      $port = isset($uri['port']) ? $uri['port'] : 80;
      $host = $uri['host'] . ($port != 80 ? ':'. $port : '');
      $fp = @fsockopen($uri['host'], $port, $errno, $errstr, 15);
      break;
    default:
      $result->error = t('Invalid schema @schema in URL @url', array('@url' => $url, '@schema' => $uri['scheme']));
      $result->code = -1003;
      return $result;
  }

  // Make sure the socket opened properly.
  if (!$fp) {
    // When a network error occurs, we use a negative number so it does not
    // clash with the HTTP status codes.
    $result->code = -$errno;
    $result->error = trim($errstr);

    // Mark that this request failed. This will trigger a check of the web
    // server's ability to make outgoing HTTP requests the next time that
    // requirements checking is performed.
    // @see system_requirements()
    global $_SESSION;
    $_SESSION['drupal_http_request_fails'] = TRUE;
    return $result;
  }

  // Construct the path to act on.
  $path = isset($uri['path']) ? $uri['path'] : '/';
  if (isset($uri['query'])) {
    $path .= '?'. $uri['query'];
  }

  // Create HTTP request.
  $defaults = array(
    // RFC 2616: "non-standard ports MUST, default ports MAY be included".
    // We don't add the port to prevent from breaking rewrite rules checking the
    // host that do not take into account the port number.
    'Host' => "Host: $host",
    'User-Agent' => 'User-Agent: Drupal (+http://drupal.org/)',
  );

  // Only add Content-Length if we actually have any content or if it is a POST
  // or PUT request. Some non-standard servers get confused by Content-Length in
  // at least HEAD/GET requests, and Squid always requires Content-Length in
  // POST/PUT requests.
  $content_length = strlen($data);
  if ($content_length > 0 || $method == 'POST' || $method == 'PUT') {
    $defaults['Content-Length'] = 'Content-Length: '. $content_length;
  }

  // If the server url has a user then attempt to use basic authentication
  if (isset($uri['user'])) {
    $defaults['Authorization'] = 'Authorization: Basic '. base64_encode($uri['user'] . (!empty($uri['pass']) ? ":". $uri['pass'] : ''));
  }

  foreach ($headers as $header => $value) {
    $defaults[$header] = $header .': '. $value;
  }

  $request = $method .' '. $path ." HTTP/1.0\r\n";
  $request .= implode("\r\n", $defaults);
  $request .= "\r\n\r\n";
  $request .= $data;

  $result->request = $request;

  fwrite($fp, $request);

  // Fetch response.
  $response = '';
  while (!feof($fp) && $chunk = fread($fp, 1024)) {
    $response .= $chunk;
  }
  fclose($fp);

  // Parse response.
  list($split, $result->data) = explode("\r\n\r\n", $response, 2);
  $split = preg_split("/\r\n|\n|\r/", $split);

  list($protocol, $code, $text) = explode(' ', trim(array_shift($split)), 3);
  $result->headers = array();

  // Parse headers.
  while ($line = trim(array_shift($split))) {
    list($header, $value) = explode(':', $line, 2);
    if (isset($result->headers[$header]) && $header == 'Set-Cookie') {
      // RFC 2109: the Set-Cookie response header comprises the token Set-
      // Cookie:, followed by a comma-separated list of one or more cookies.
      $result->headers[$header] .= ','. trim($value);
    }
    else {
      $result->headers[$header] = trim($value);
    }
  }

  $responses = array(
    100 => 'Continue', 101 => 'Switching Protocols',
    200 => 'OK', 201 => 'Created', 202 => 'Accepted', 203 => 'Non-Authoritative Information', 204 => 'No Content', 205 => 'Reset Content', 206 => 'Partial Content',
    300 => 'Multiple Choices', 301 => 'Moved Permanently', 302 => 'Found', 303 => 'See Other', 304 => 'Not Modified', 305 => 'Use Proxy', 307 => 'Temporary Redirect',
    400 => 'Bad Request', 401 => 'Unauthorized', 402 => 'Payment Required', 403 => 'Forbidden', 404 => 'Not Found', 405 => 'Method Not Allowed', 406 => 'Not Acceptable', 
    407 => 'Proxy Authentication Required', 408 => 'Request Time-out', 409 => 'Conflict', 410 => 'Gone', 411 => 'Length Required', 412 => 'Precondition Failed', 
    413 => 'Request Entity Too Large', 414 => 'Request-URI Too Large', 415 => 'Unsupported Media Type', 416 => 'Requested range not satisfiable', 417 => 'Expectation Failed',
    500 => 'Internal Server Error', 501 => 'Not Implemented', 502 => 'Bad Gateway', 503 => 'Service Unavailable', 504 => 'Gateway Time-out', 505 => 'HTTP Version not supported'
  );
  // RFC 2616 states that all unknown HTTP codes must be treated the same as the
  // base code in their class.
  if (!isset($responses[$code])) {
    $code = floor($code / 100) * 100;
  }

  switch ($code) {
    case 200: // OK
    case 304: // Not modified
      break;
    case 301: // Moved permanently
    case 302: // Moved temporarily
    case 307: // Moved temporarily
      $location = $result->headers['Location'];

      if ($retry) {
        $result = drupal_http_request($result->headers['Location'], $headers, $method, $data, --$retry);
        $result->redirect_code = $result->code;
      }
      $result->redirect_url = $location;

      break;
    default:
      $result->error = $text;
  }

  $result->code = $code;
  return $result;
}

/**
 * Gets the PHP environment information.
 * You won't believe how this is done...
 * @see http://www.php.net/manual/en/function.phpinfo.php#87463
 *
 * @return an array containing the information.
 */
function phpinfo_array() {
  ob_start();
  phpinfo(-1);
 
  $elements = preg_replace(
    array(
      '#^.*<body>(.*)</body>.*$#ms',
      '#<h2>PHP License</h2>.*$#ms',
      '#<h1>Configuration</h1>#',
      "#\r?\n#",
      "#</(h1|h2|h3|tr)>#",
      '# +<#',
      "#[ \t]+#",
      '#&nbsp;#',
      '#  +#',
      '# class=".*?"#',
      '%&#039;%',
      '#<tr>(?:.*?)' . '<h1>PHP Version (.*?)</h1>(?:\n+?)</td></tr>#',
      '#<h1><a href="(?:.*?)\?=(.*?)">PHP Credits</a></h1>#',
      '#<tr>(?:.*?)" src="(?:.*?)=(.*?)"(?:.*?)Zend Engine (.*?),(?:.*?)</tr>#',
      "# +#",
      '#<tr>#',
      '#</tr>#'
    ),
    array(
      '$1', 
      '', 
      '', 
      '',
      '</$1>' . "\n",
      '<',
      ' ',
      ' ',
      ' ',
      '',
      ' ',
      '<h2>PHP Configuration</h2>'."\n".'<tr><td>PHP Version</td><td>$1</td></tr>',
      '<tr><td>PHP Credits Egg</td><td>$1</td></tr>',
      '<tr><td>Zend Engine</td><td>$2</td></tr>'."\n".'<tr><td>Zend Egg</td><td>$1</td></tr>',
      ' ',
      '%S%',
      '%E%'
    ),
    ob_get_clean()
  );

  $sections = explode('<h2>', strip_tags($elements, '<h2><th><td>'));
  unset($sections[0]);

  $elements = array();
  foreach ($sections as $section) {
    $name = substr($section, 0, strpos($section, '</h2>'));
    preg_match_all(
      '#%S%(?:<td>(.*?)</td>)?(?:<td>(.*?)</td>)?(?:<td>(.*?)</td>)?%E%#',
      $section, $pieces, PREG_SET_ORDER);
    foreach ($pieces as $piece) {
      $elements[$name][$piece[1]] = (!isset($piece[3]) || $piece[2] == $piece[3]) ? $piece[2] : array_slice($piece, 2);
    }
  }

 return $elements;
 }

/**
 * Hosting Tests specific code starts here
 */

/**
 * Sets result messages for a which reflects the status of the performed operation.
 *
 * @param $title
 *   The test title.
 * @param $status
 *   The array of status messages.
 * @param $warning
 *   The array of warning messages.
 * @param $error
 *   The array of error messages.
 * @return
 *   The report class.
 */
function set_results($title, $status = array(), $warning = array(), $error = array()) {
  $report = new stdClass();

  $report->title = $title;
  $report->status = $status;
  $report->warning = $warning;
  $report->error = $error;
  return $report;
}

/**
 * Perform checks about PHP version.
 *
 * Lifted from modules/system/system.install
 *
 * @return the test results
 */
function php_version_check() {
  $results = set_results(t('PHP %version check', array("%version" => 'phpversion()')));
  if (version_compare(phpversion(), DRUPAL_MINIMUM_PHP) < 0) {
    $results->error[] = t('Your PHP installation is too old. Drupal requires at least PHP %version.', array('%version' => DRUPAL_MINIMUM_PHP));
  }
  else {
    $results->status[] = t("PHP %version acceptable.", array("%version" => 'phpversion()'));
  }
  return $results;
}


/**
 * Perform checks about PHP memory_limit
 *
 * Lifted from modules/system/system.install
 *
 * @return the test results
 */
function php_memory_limit_check() {
  $memory_limit = ini_get('memory_limit');
  $results = set_results(t('PHP %memory_limit check', array("%memory_limit" => 'memory_limit')));

  if ($memory_limit && $memory_limit != -1 && parse_size($memory_limit) < parse_size(DRUPAL_MINIMUM_PHP_MEMORY_LIMIT)) {
    $results->error[] = t('Depending on your configuration, Drupal can run with a %memory_limit PHP memory limit. However, a %memory_minimum_limit PHP memory limit or above is recommended, especially if your site uses additional custom or contributed modules.', array('%memory_limit' => $memory_limit, '%memory_minimum_limit' => DRUPAL_MINIMUM_PHP_MEMORY_LIMIT));
  }
  else {
    $results->status[] = t("PHP %memory_limit acceptable.", array("%memory_limit" => 'memory_limit'));
  }
  return $results;
}

/**
 * Perform checks about PHP register_globals
 *
 * Lifted from modules/system/system.install
 *
 * @return the test results
 */
function php_register_globals_check() {
  $register_globals = trim(ini_get('register_globals'));
  $results = set_results(t('PHP %register_globals check', array("%register_globals" => 'register_globals')));
  // Unfortunately, ini_get() may return many different values, and we can't
  // be certain which values mean 'on', so we instead check for 'not off'
  // since we never want to tell the user that their site is secure
  // (register_globals off), when it is in fact on. We can only guarantee
  // register_globals is off if the value returned is 'off', '', or 0.
  if (!empty($register_globals) && strtolower($register_globals) != 'off') {
    $results->error[] = t('%register_globals is enabled. Drupal requires this configuration directive to be disabled. Your site may not be secure when %register_globals is enabled. The PHP manual has instructions for <a href="http://php.net/configuration.changes">how to change configuration settings</a>.', array("%register_globals" => 'register_globals'));
  }
  else {
    $results->status[] = t("PHP %register_globals acceptable.", array("%register_globals" => 'register_globals'));
  }
  return $results;
}

/**
 * Perform checks about PHP safe_mode
 *
 * @return the test results
 */
function php_safe_mode_check() {
  $safe_mode = trim(ini_get('safe_mode'));
  $results = set_results(t('PHP %safe_mode check', array("%safe_mode" => 'safe_mode')));
  // Unfortunately, ini_get() may return many different values, and we can't
  // be certain which values mean 'on', so we instead check for 'not off'
  // since we never want to tell the user that their site is secure
  // (safe_mode off), when it is in fact on. We can only guarantee
  // safe_mode is off if the value returned is 'off', '', or 0.
  if (!empty($safe_mode) && strtolower($safe_mode) != 'off') {
    $results->error[] = t('%safe_mode is enabled. Drupal requires this configuration directive to be disabled. Your site may have problems uploading files when %safe_mode is enabled. The PHP manual has instructions for <a href="http://php.net/configuration.changes">how to change configuration settings</a>.', array("%safe_mode" => 'safe_mode'));
  }
  else {
    $results->status[] = t("PHP %safe_mode acceptable.", array("%safe_mode" => 'safe_mode'));
  }
  return $results;
}

/**
 * Perform checks about Unicode support in PHP.
 *
 * Because Drupal needs to be able to handle text in various encodings, we do
 * not support mbstring function overloading. HTTP input/output conversion must
 * be disabled for similar reasons.
 *
 * Lifted from includes/unicode.inc
 *
 * @return the test results
 */
function unicode_check() {
  $results = set_results(t('PHP Unicode support check'));
  $fail = false;

  // Set the standard C locale to ensure consistent, ASCII-only string handling.
  setlocale(LC_CTYPE, 'C');

  // Check for outdated PCRE library
  // Note: we check if U+E2 is in the range U+E0 - U+E1. This test returns TRUE on old PCRE versions.
  if (preg_match('/[à-á]/u', 'â')) {
    $fail = true;
    $results->error[] = t('The PCRE library in your PHP installation is outdated. This will cause problems when handling Unicode text. If you are running PHP 4.3.3 or higher, make sure you are using the PCRE library supplied by PHP. Please refer to the <a href="@url">PHP PCRE documentation</a> for more information.', array('@url' => 'http://www.php.net/pcre'));
  }

  // Check for mbstring extension
  if (!function_exists('mb_strlen')) {
    $fail = true;
    $results->warning[] = t('Operations on Unicode strings are emulated on a best-effort basis. Install the <a href="@url">PHP mbstring extension</a> for improved Unicode support.', array('@url' => 'http://www.php.net/mbstring'));
  }
  else {
    // Check mbstring configuration
    if (ini_get('mbstring.func_overload') != 0) {
      $fail = true;
      $results->error[] = t('Multibyte string function overloading in PHP is active and must be disabled. Check the php.ini <em>mbstring.func_overload</em> setting. Please refer to the <a href="@url">PHP mbstring documentation</a> for more information.', array('@url' => 'http://www.php.net/mbstring'));
    }
    if (ini_get('mbstring.encoding_translation') != 0) {
      $fail = true;
      $results->error[] = t('Multibyte string input conversion in PHP is active and must be disabled. Check the php.ini <em>mbstring.encoding_translation</em> setting. Please refer to the <a href="@url">PHP mbstring documentation</a> for more information.', array('@url' => 'http://www.php.net/mbstring'));
    }
    if (ini_get('mbstring.http_input') != 'pass') {
      $fail = true;
      $results->error[] = t('Multibyte string input conversion in PHP is active and must be disabled. Check the php.ini <em>mbstring.http_input</em> setting. Please refer to the <a href="@url">PHP mbstring documentation</a> for more information.', array('@url' => 'http://www.php.net/mbstring'));
    }
    if (ini_get('mbstring.http_output') != 'pass') {
      $fail = true;
      $results->error[] = t('Multibyte string output conversion in PHP is active and must be disabled. Check the php.ini <em>mbstring.http_output</em> setting. Please refer to the <a href="@url">PHP mbstring documentation</a> for more information.', array('@url' => 'http://www.php.net/mbstring'));
    }
  }

  if ($fail == false) {
    $results->status[] = t("PHP Unicode support acceptable.");
  }
  return $results;
}

/**
 * Perform checks about GD
 *
 * Lifted from sites/all/modules/imageapi/imageapi_gd.install
 *
 * @return the test results
 */
function imageapi_check() {
  $results = set_results(t('PHP (ImageApi) GD check'));
  $fail = false;
  $gd = function_exists('imagegd2');

  if (!$gd) {
    $fail = true;
    $results->error[] = t('The GD library for PHP is missing or outdated. Please check the <a href="@url">PHP image documentation</a> for information on how to correct this.', array('@url' => 'http://www.php.net/manual/en/image.setup.php'));
  } 
  else {
    // report version.
    $info = gd_info();
    $results->status[] = t('GD library: %version.', array('%version' => $info['GD Version']));

    // Check image format support 
    foreach (array('gif', 'jpeg', 'png') as $format) {
      if (function_exists('imagecreatefrom'. $format)) continue;
      $fail = true;
      $results->error[] = t('PHP GD was not compiled with %format support.', array('%format' => $format));
    }

    if (!function_exists('imagerotate')) {
      $fail = true;
      $results->warning[] = t('The installed version of PHP GD does not support image rotations. It was probably compiled using the official GD libraries from http://www.libgd.org instead of the GD library bundled with PHP. You should recompile PHP --with-gd using the bundled GD library. See: @url. An implementation of imagerotate in PHP will used in the interim.',  array('@url' => 'http://www.php.net/manual/en/image.setup.php'));
    }

    if (!function_exists('imagefilter')) {
      $fail = true;
      $results->warning[] = t('The installed version of PHP GD does not support image filtering (desaturate, blur, negate, etc). It was probably compiled using the official GD libraries from http://www.libgd.org instead of the GD library bundled with PHP. You should recompile PHP --with-gd using the bundled GD library. See @url. An implementation of imagefilter in PHP will be used in the interim.',  array('@url' => 'http://www.php.net/manual/en/image.setup.php'));
    }

    $memory_limit = ini_get('memory_limit');
    
    if (parse_size($memory_limit) < parse_size('96M')) {
      $fail = true;
      $results->warning[] = t('It is highly recommended that you set you PHP memory_limit to 96M to use ImageAPI GD. A 1600x1200 images consumes ~45M of memory when decompressed and there are instances where ImageAPI GD is operating on two decompressed images at once.');
    }
  }
  if ($fail == false) {
    $results->status[] = t("PHP (ImageApi) GD support acceptable.");
  }
  return $results;
}

/**
 * Perform checks about GD
 *
 * Lifted from sites/all/modules/filefield/filefield.install
 *
 * @return the test results
 */
function upload_progress_check() {
  $results = set_results(t('Upload progress check'));

  $implementation = FALSE;

  // We prefer the PECL extension uploadprogress because it supports multiple
  // simultaneous uploads. APC only supports one at a time.
  if (extension_loaded('uploadprogress')) {
    $implementation = 'uploadprogress';
  }
  elseif (extension_loaded('apc') && ini_get('apc.rfc1867')) {
    $implementation = 'apc';
  }

  $apache = strpos($_SERVER['SERVER_SOFTWARE'], 'Apache') !== FALSE;
  $fastcgi = strpos($_SERVER['SERVER_SOFTWARE'], 'mod_fastcgi') !== FALSE || strpos($_SERVER["SERVER_SOFTWARE"], 'mod_fcgi') !== FALSE;
  $php_52 = version_compare(phpversion(), '5.2.0', '>');
  if (!$apache || !$php_52) {
    $results->warning[] = t('Your server is not capable of displaying file upload progress. File upload progress requires PHP 5.2 and an Apache server.');
  }
  elseif ($fastcgi) {
    $results->warning[] = t('Your server is not capable of displaying file upload progress. File upload progress requires PHP be run with mod_php and not as FastCGI.');
  }
  elseif (!$implementation && extension_loaded('apc')) {
    $results->warning[] = t('Your server is capable of displaying file upload progress through APC, but it is not enabled. Add <code>apc.rfc1867 = 1</code> to your php.ini configuration. Alternatively, it is recommended to use <a href="http://pecl.php.net/package/uploadprogress">PECL uploadprogress</a>, which supports more than one simultaneous upload.');
  }
  elseif (!$implementation) {
    $results->warning[] = t('Your server is capable of displaying file upload progress, but does not have the required libraries. It is recommended to install the <a href="http://pecl.php.net/package/uploadprogress">PECL uploadprogress library</a> (preferred) or to install <a href="http://us2.php.net/apc">APC</a>.');
  }
  elseif ($implementation == 'apc') {
    $results->status[] = t('Your server is capable of displaying file upload progress using APC RFC1867. Note that only one upload at a time is supported. It is recommended to use the <a href="http://pecl.php.net/package/uploadprogress">PECL uploadprogress library</a> if possible.');
  }
  elseif ($implementation == 'uploadprogress') {
    $results->status[] = t('Enabled (<a href="http://pecl.php.net/package/uploadprogress">PECL uploadprogress</a>)');
  }
  else {
    $results->warning[] = t('Your server is not capable of displaying file upload progress. File upload progress requires PHP 5.2 and an Apache server.');
  }
  return $results;
}

/**
 * Perform checks about PHP settings override in .htaccess
 *
 * Not lifted from anywhere!
 *
 * @return the test results
 */
function php_settings_override_check() {
  $results = set_results(t('Overridden PHP settings (<em>.htaccess</em>) check'));
  $hosting_test = ini_get('error_prepend_string');

  if ($hosting_test != "Hosting test: ") {
    $results->warning[] = t("Overridden values of PHP settings not accepted.");
  }
  else {
    $results->status[] = t("Overridden PHP settings accepted.");
  }
  return $results;
}

/**
 * Perform checks about PHP php.ini settings override
 *
 * Not lifted from anywhere!
 *
 * @return the test results
 */
function php_ini_settings_override_check() {
  $results = set_results(t('Overridden PHP settings (<em>php.ini</em>) check'));
  $hosting_test = ini_get('error_append_string');

  if ($hosting_test != " :stset gnitsoH") {
    $results->warning[] = t("Overridden values of PHP settings not accepted.");
  }
  else {
    $results->status[] = t("Overridden PHP settings accepted.");
  }
  return $results;
}

/**
 * Perform checks about PHP ini_set() settings override
 * @see http://www.php.net/manual/en/configuration.changes.modes.php
 * @see http://www.php.net/manual/en/ini.list.php
 *
 * Not lifted from anywhere!
 *
 * @return the test results
 */
function php_ini_set_override_check() {
  $results = set_results(t('Overridden PHP settings (<em>ini_set()</em>) check'));
  
  $memory_limit = ini_get('memory_limit');
  $new_limit = $memory_limit == '32M' ? '31M' : '32M';
  ini_set('memory_limit', $new_limit);

  $hosting_test = ini_get('memory_limit');
  if ($hosting_test != $new_limit) {
    $results->error[] = t("Overridden values of PHP settings not accepted.");
  }
  else {
    $results->status[] = t("Overridden PHP settings accepted. If all other PHP settings overrides fail, consider using <a href=\"http://drupal.org/project/drupal_tweaks\">Drupal Tweaks</a>.");
    ini_set('memory_limit', $memory_limit);
  }
  return $results;
}

/**
 * Perform checks about the files directory
 *
 * Lifted from modules/system/system.module
 *
 * @param $dir the directory to check
 * @return the test results
 */
function files_directory_check($dir) {
  $results = set_results(t('Directory %dir check', array('%dir' => $dir)));

  // not quite sure this is the best way to do this...
  $path = ''. __FILE__;
  $delimiter = strpos($path, '/') === false ? '\\' : '/';
  $path = explode($delimiter, $path);
  unset($path[count($path) - 1]);
  $path[] = $dir;
  $directory = implode($delimiter, $path);
  $display_dir = strstr($directory, 'hosting-tests'); // be discreet

  $is_directory = is_dir($directory);
  $is_readable = is_readable($directory);
  $is_writable = is_writable($directory);
  $is_traversable = is_executable($directory);
  if (!$is_readable || !$is_writable || !$is_traversable || !$is_directory) {
    if (!$is_directory) {
      $results->error[] = t('The directory %directory does not exist. (This shouldn\'t happen)', array('%directory' => $display_dir));
    }
    else {
      if (!$is_readable) {
        $results->error[] = t('The directory %directory is not readable.', array('%directory' => $display_dir));
      }
      if (!$is_writable) {
        $results->error[] = t('The directory %directory is not writable.', array('%directory' => $display_dir));
      }
      if (!$is_traversable) {
        $results->error[] = t('The directory %directory is not traversable.', array('%directory' => $display_dir));
      }
    }
  }
  else {
    $results->status[] = t('The directory %directory exists, is readable, writable, and traversable.', array('%directory' => $display_dir));
  }
  return $results;
}

/**
 * Checks whether the server is capable of performing rewrite rules.
 *
 * Not lifted from anywhere!
 *
 * @param $php_info the PHP information.
 * @return the test results
 */
function server_mod_rewrite_check($php_info) {
  $results = set_results(t('Server rewrite module check'));
  $modules = $php_info['apache2handler']['Loaded Modules'];
  if (!isset($modules)) {
    $results->warning[] = t("No server information found.");
  }
  else {
    if (!strstr($modules, 'mod_rewrite')) {
      $results->error[] = t("No rewrite module installed on the server.");
    }
    else {
      $results->status[] = t("The rewrite module is installed.");
    }
  }

  return $results;
}


/**
 * Since $_SERVER['REQUEST_URI'] is only available on Apache, we
 * generate an equivalent using other environment variables.
 *
 * Lifted from includes/bootstrap.inc
 */
function request_uri() {
  if (isset($_SERVER['REQUEST_URI'])) {
    $uri = $_SERVER['REQUEST_URI'];
  }
  else {
    if (isset($_SERVER['argv'])) {
      $uri = $_SERVER['SCRIPT_NAME'] .'?'. $_SERVER['argv'][0];
    }
    elseif (isset($_SERVER['QUERY_STRING'])) {
      $uri = $_SERVER['SCRIPT_NAME'] .'?'. $_SERVER['QUERY_STRING'];
    }
    else {
      $uri = $_SERVER['SCRIPT_NAME'];
    }
  }
  // Prevent multiple slashes to avoid cross site requests via the FAPI.
  $uri = '/'. ltrim($uri, '/');

  return $uri;
}

/**
 * Checks whether the server is capable of issuing HTTP requests.
 *
 * The function sets the drupal_http_request_fail system variable to TRUE if
 * drupal_http_request() does not work and then the system status report page
 * will contain an error.
 *
 * Lifted from modules/system/system.module
 *
 * @param $url the current URL.
 * @return the test results
 */
function http_request_check($url) {
  global $_SESSION;
  $_SESSION['drupal_http_request_fails'] = FALSE;
  $results = set_results(t('HTTP request check'));
  $results->status[] = t('HTTP request: %url', array('%url' => 'hosting-tests/favicon.png'));

  // make sure the $url value is reasonable
  $url_valid = TRUE;
  $domain_pos = strpos($url, '://');
  if ($domain_pos !== FALSE) {
    $path = substr($url, strrpos($url, '/') + 1);
    if ($path !== 'hosting-tests') {
      $url_valid = FALSE;
    }
  }
  else {
    $url_valid = FALSE;
  }

  if ($url_valid == FALSE) {
    $_SESSION['drupal_http_request_fails'] = TRUE;
    $results->error[] = t("HTTP request failed - incorrect URL: expecting '%url'", array('%url' => 'http(s)://domain(/path)/hosting-tests'));
  }
  else {  
    $request = $url .'/favicon.png';
    $response = drupal_http_request($request);
    // We only care that we get a http response - this means that Drupal can make a http request.
    $works = isset($response->code) && ($response->code >= 100) && ($response->code < 600);
    if (!$works) {
      $_SESSION['drupal_http_request_fails'] = TRUE;
      $results->error[] = t("HTTP request failed - error code: !code.", array('!code' => $response->code));
    }
    else {
      $results->status[] = t("HTTP request successful - code: !code.", array('!code' => $response->code));
    }
  }
  return $results;
}

/**
 * Checks whether the server is capable of issuing external HTTP requests.
 *
 * Not lifted from anywhere!
 *
 * @return the test results
 */
function external_http_request_check() {
  $results = set_results(t('External HTTP request check'));
  $request = 'http://updates.drupal.org/release-history/drupal/6.x';
  $results->status[] = t('External HTTP request: %url', array('%url' => $request));
  $response = drupal_http_request($request);
  $works = isset($response->code) && ($response->code == 200);
  if (!$works) {
    $results->error[] = t("External HTTP request failed - error code: !code.", array('!code' => $response->code));
  }
  else {
    $results->status[] = t("External HTTP request successful - code: !code.", array('!code' => $response->code));
  }
  return $results;
}

/**
 * Checks whether the server is capable of rewriting URLs.
 *
 * @param $url the current URL.
 * @return the test results
 */
function server_rewrite_check($url) {
  $results = set_results(t('Server rewrite check'));
  $results->status[] = t('HTTP request: %url', array('%url' => 'hosting-tests/link2?s=1'));

  $request = $url .'/link2?s=1';
  $response = drupal_http_request($request);
  $works = isset($response->code) && ($response->code == 200);
  if (!$works) {
    $results->error[] = t("Server rewrite failed - error code: !code.", array('!code' => $response->code));
  }
  else {
    if ($response->data != 'link2') {
      $results->error[] = t("Server rewrite failed - error code: !code, data: '@data'.", array('!code' => $response->code, '@data' => $response->data));
    }
    else {
      $results->status[] = t("Server rewrite successful - code: !code.", array('!code' => $response->code));
    }
  }
  return $results;
}

/**
 * Checks whether the server is redirecting 404 errors to us.
 *
 * @param $url the current URL.
 * @return the test results
 */
function missing_redirect_check($url) {
  $results = set_results(t('Missing (404) redirect check'));
  $results->status[] = t('HTTP request: %url', array('%url' => 'hosting-tests/system/files'));

  $request = $url .'/system/files';
  $response = drupal_http_request($request);
  $works = isset($response->code) && ($response->code == 404);
  if (!$works) {
    $results->error[] = t("Missing (404) redirect failed - error code: !code.", array('!code' => $response->code));
  }
  else {
    if ($response->data != "You aren't allowed to see this") {
      $results->error[] = t("Missing (404) redirect failed - error code: !code, data is not '@data'.", array('!code' => $response->code, '@data' => "You aren't allowed to see this"));
    }
    else {
      $results->status[] = t("Missing (404) redirect successful - code: !code, data: '@data'.", array('!code' => $response->code, '@data' => $response->data));
    }
  }
  return $results;
}

/**
 * Checks whether the server returns an image in one of the protected directories.
 *
 * @param $url the current URL.
 * @param $dir the protected directory.
 * @return the test results
 */
function retrieve_files_image_check($url, $dir) {
  $results = set_results(t('Retrieve image in %dir directory check', array('%dir' => $dir)));
  $results->status[] = t('HTTP request: %url', array('%url' => 'hosting-tests/'. $dir .'/drupalitalia.png'));

  $request = $url .'/'. $dir .'/drupalitalia.png';
  $response = drupal_http_request($request);
  $works = isset($response->code) && ($response->code == 200);
  if (!$works) {
    $results->error[] = t("Retrieve image failed - error code: !code.", array('!code' => $response->code));
  }
  else {
    if (strlen($response->data) != 5605) {
      $results->error[] = t("Retrieve image failed - error code: !code.", array('!code' => $response->code));
    }
    else {
      $results->status[] = t("Retrieve image successful - code: !code, size: !size.", array('!code' => $response->code, '!size' => strlen($response->data)));
    }
  }
  return $results;
}

/**
 * Checks whether the server returns an unexecuted script file in one of the protected directories.
 *
 * @param $url the current URL.
 * @param $dir the protected directory.
 * @return the test results
 */
function retrieve_files_script_check($url, $dir) {
  $results = set_results(t('Retrieve script in %dir directory check', array('%dir' => $dir)));
  $results->status[] = t('HTTP request: %url', array('%url' => 'hosting-tests/'. $dir .'/script.php?s=1'));

  $request = $url .'/'. $dir .'/script.php?s=1';
  $response = drupal_http_request($request);
  $works = isset($response->code) && ($response->code == 200);
  if (!$works) {
    $results->error[] = t("Retrieve script failed - error code: !code.", array('!code' => $response->code));
  }
  else {
    // $results->status[] = t("'@data'", array('@data' => $response->data));
    if (strlen($response->data) == 'This script should not be executed') {
      $results->error[] = t("Retrieve script was executed - code: !code data '@data'.", array('!code' => $response->code, '@data' => $response->data));
    }
    else {
      $results->status[] = t("Retrieve script successful - code: !code, size: !size.", array('!code' => $response->code, '!size' => strlen($response->data)));
    }
  }
  return $results;
}

/**
 * Checks whether the server returns a list of one of the protected directories.
 *
 * @param $url the current URL.
 * @param $dir the protected directory.
 * @return the test results
 */
function retrieve_files_dir_list_check($url, $dir) {
  $results = set_results(t('Directory %dir listing protection check', array('%dir' => $dir)));
  $results->status[] = t('HTTP request: %url', array('%url' => 'hosting-tests/'. $dir .'/'));

  $request = $url .'/'. $dir .'/';
  $response = drupal_http_request($request);
  $works = isset($response->code) && ($response->code >= 200 && $response->code <= 299);
  if ($works) {
    $results->error[] = t("Directory listing was created - code: !code.", array('!code' => $response->code));
  }
  else {
    if ($response->code && ($response->code == 403 || $response->code == 404)) {
      $results->status[] = t("Directory listing protection successful - code: !code.", array('!code' => $response->code));
    }
    else {
      $results->error[] = t("Directory listing failed with error code: !code.", array('!code' => $response->code));
    }
  }
  return $results;
}

/**
 * Get the URL of this script.
 *
 * Lifted from includes/bootstrap.inc
 *
 * @return the fully qualified script URL
 */
function get_script_url() {
  if (isset($_SERVER['HTTP_HOST'])) {
    // As HTTP_HOST is user input, ensure it only contains characters allowed
    // in hostnames. See RFC 952 (and RFC 2181).
    // $_SERVER['HTTP_HOST'] is lowercased here per specifications.
    $_SERVER['HTTP_HOST'] = strtolower($_SERVER['HTTP_HOST']);
  }
  else {
    // Some pre-HTTP/1.1 clients will not send a Host header. Ensure the key is
    // defined for E_ALL compliance.
    $_SERVER['HTTP_HOST'] = '';
  }

  // Create base URL
  $base_root = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
  $base_url = $base_root .= '://'. $_SERVER['HTTP_HOST'];

  // $_SERVER['SCRIPT_NAME'] can, in contrast to $_SERVER['PHP_SELF'], not
  // be modified by a visitor.
  if ($dir = trim(dirname($_SERVER['SCRIPT_NAME']), '\,/')) {
    $base_path = "/$dir";
    $base_url .= $base_path;
    $base_path .= '/';
  }
  else {
    $base_path = '/';
  }

  return $base_url;
}

/**
 * Perform all tests.
 *
 * @return the report.
 */
function run_tests() {
  $report = array();
  $php_info = phpinfo_array();
  $base_url = get_script_url();

  // start with the easy stuff...
  $report['environment']['System'] = $results = set_results(t('System'));
  $results->status[] = $php_info['PHP Configuration']['System'];
  $report['environment']['Web Server'] = $results = set_results(t('Web Server'));
  $results->status[] = $_SERVER['SERVER_SOFTWARE'];
  $report['environment']['Server API'] = $results = set_results(t('Server API'));
  $results->status[] = $php_info['PHP Configuration']['Server API'];
  $report['environment']['PHP Version'] = $results = set_results(t('PHP Version'));
  $results->status[] = phpversion();
  //$report['environment']['Thread safety'] = $results = set_results(t('Thread safety'));
  //$results->status[] = $php_info['PHP Configuration']['Thread Safety'];
  //$report['environment']['Registered Stream Socket Transports'] = $results = set_results(t('Registered Stream Socket Transports'));
  //$results->status[] = $php_info['PHP Configuration']['Registered Stream Socket Transports'];
  $report['environment']['PHP memory_limit'] = $results = set_results(t('PHP <em>memory_limit</em>'));
  $results->status[] = $php_info['PHP Core']['memory_limit'];
  //$report['environment']['Allow URL fopen()'] = $results = set_results(t('Allow URL <em>fopen()</em>'));
  //$results->status[] = $php_info['PHP Core']['allow_url_fopen'];
  $report['environment']['PHP register_globals'] = $results = set_results(t('PHP <em>register_globals</em>'));
  $results->status[] = $php_info['PHP Core']['register_globals'];
  $report['environment']['PHP safe_mode'] = $results = set_results(t('PHP <em>safe_mode</em>'));
  $results->status[] = $php_info['PHP Core']['safe_mode'];

  $report['environment']['MySQL Client API version'] = $results = set_results(t('MySQL Client API version'));
  $version = $php_info['mysql']['Client API version'];
  $results->status[] = $version ? $version : '-';
  $report['environment']['MySQLi Client API version'] = $results = set_results(t('MySQL<em>i</em> Client API version'));
  $version = $php_info['mysqli']['Client API library version'];
  $results->status[] = $version ? $version : '-';
  $report['environment']['PostgreSQL Version'] = $results = set_results(t('PostgreSQL Version'));
  $version = $php_info['pgsql']['PostgreSQL(libpq) Version'];
  $results->status[] = $version ? $version : '-';

  // then do the difficult stuff...
  $report['req_tests'][] = php_version_check();
  $report['req_tests'][] = php_memory_limit_check();
  $report['req_tests'][] = php_register_globals_check();
  $report['req_tests'][] = php_safe_mode_check();
  $report['req_tests'][] = unicode_check();
  $report['req_tests'][] = imageapi_check();
  $report['req_tests'][] = upload_progress_check();
  $report['req_tests'][] = php_settings_override_check();
  $report['req_tests'][] = php_ini_settings_override_check();
  $report['req_tests'][] = php_ini_set_override_check();
  $report['req_tests'][] = files_directory_check('files');
  $report['req_tests'][] = files_directory_check('files-alt');
  $report['req_tests'][] = server_mod_rewrite_check($php_info);
  $report['req_tests'][] = external_http_request_check();
  $report['req_tests'][] = http_request_check($base_url);
  global $_SESSIONS;
  if ($_SESSION['drupal_http_request_fails'] == FALSE) {
    // proceed with automatic tests
    $report['req_tests'][] = server_rewrite_check($base_url);
    $report['req_tests'][] = missing_redirect_check($base_url);
    $report['req_tests'][] = retrieve_files_image_check($base_url, 'files');
    $report['req_tests'][] = retrieve_files_script_check($base_url, 'files');
    $report['req_tests'][] = retrieve_files_dir_list_check($base_url, 'files');
    $report['req_tests'][] = retrieve_files_image_check($base_url, 'files-alt');
    $report['req_tests'][] = retrieve_files_script_check($base_url, 'files-alt');
    $report['req_tests'][] = retrieve_files_dir_list_check($base_url, 'files-alt');

    $report['reports'][] = $results = set_results(t('Plain text report'));
    $request = $base_url .'/plain-text.php';
    $results->status[] = t("Use this <a href='@url'>link</a> to see the plain text report.", array('@url' => $request));
  }
  else {
    // no http_request - no party... proceed with manual tests
    $report['req_tests'][] = $results = set_results(t('Server rewrite check'));
    $request = $base_url .'/link2?s=1';
    $results->warning[] = t("The check cannot be tested automatically. Use this <a href='@url'>link</a> to test manually.", array('@url' => $request));
    $results->error[] = t("The test succeeds if a page similar to this is displayed showing a success message, <strong>otherwise the test fails</strong>."); 

    $report['req_tests'][] = $results = set_results(t('Missing (404) redirect check'));
    $request = $base_url .'/system/files';
    $results->warning[] = t("The check cannot be tested automatically. Use this <a href='@url'>link</a> to test manually.", array('@url' => $request));
    $results->error[] = t("The test succeeds if a page is displayed showing a message \"You aren't allowed to see this\", <strong>otherwise the test fails</strong>."); 

    $report['req_tests'][] = $results = set_results(t('Retrieve image in <em>files</em> directory check'));
    $request = $base_url .'/files/drupalitalia.png';
    $results->warning[] = t("The check cannot be tested automatically. Use this <a href='@url'>link</a> to test manually.", array('@url' => $request));
    $results->error[] = t("The test fails if you <strong>don't see an image</strong> after clicking the link."); 

    $report['req_tests'][] = $results = set_results(t('Retrieve script in <em>files</em> directory check'));
    $request = $base_url .'/files/script.php?s=1';
    $results->warning[] = t("The check cannot be tested automatically. Use this <a href='@url'>link</a> to test manually.", array('@url' => $request));
    $results->error[] = t("The test fails if the browser <strong>does not offer to download the file</strong>, but instead displays a text file with the words '<em>This script should not be executed</em>'."); 

    $report['req_tests'][] = $results = set_results(t('Directory <em>files</em> listing protection check'));
    $request = $base_url .'/files/';
    $results->warning[] = t("The check cannot be tested automatically. Use this <a href='@url'>link</a> to test manually.", array('@url' => $request));
    $results->error[] = t("The test fails if the browser <strong>provides a file listing of the directory</strong>."); 

    $report['req_tests'][] = $results = set_results(t('Retrieve image in <em>files-alt</em> directory check'));
    $request = $base_url .'/files-alt/drupalitalia.png';
    $results->warning[] = t("The check cannot be tested automatically. Use this <a href='@url'>link</a> to test manually.", array('@url' => $request));
    $results->error[] = t("The test fails if you <strong>don't see an image</strong> after clicking the link."); 

    $report['req_tests'][] = $results = set_results(t('Retrieve script in <em>files-alt</em> directory check'));
    $request = $base_url .'/files-alt/script.php?s=1';
    $results->warning[] = t("The check cannot be tested automatically. Use this <a href='@url'>link</a> to test manually.", array('@url' => $request));
    $results->error[] = t("The test fails if the browser <strong>does not offer to download the file</strong>, but instead displays a text file with the words '<em>This script should not be executed</em>'."); 

    $report['req_tests'][] = $results = set_results(t('Directory <em>files-alt</em> listing protection check'));
    $request = $base_url .'/files-alt/';
    $results->warning[] = t("The check cannot be tested automatically. Use this <a href='@url'>link</a> to test manually.", array('@url' => $request));
    $results->error[] = t("The test fails if the browser <strong>provides a file listing of the directory</strong>."); 

    $report['reports'][] = $results = set_results(t('Plain text report'));
    $request = $base_url .'/plain-text.php';
    $results->status[] = t("Use this <a href='@url'>link</a> to see the plain text report. You will have to add the results of the manual tests yourself.", array('@url' => $request));
  }

  return $report;
}

/**
 * Render the tabular report
 *
 * @param $report the report to render
 * @param $info true if information only, otherwise false.
 */
function render_tabular_report($report, $info = false) {
  print('<table><tbody>');
  foreach ($report as $result) {
    print('<tr><td class="title"><p class="title">'. $result->title .'</p></td>');
    if ($info) {
      print('<td class="info">');
    }
    else {
      print('<td class="result">');
    }

    foreach ($result->status as $status) {
      print('<p class="status">'. $status .'</p>');
    }
    foreach ($result->warning as $warning) {
      print('<p class="warning">'. $warning .'</p>');
    }
    foreach ($result->error as $error) {
      print('<p class="error">'. $error .'</p>');
    }
    print('</td>');
    print('</tr>');
  }
  print('</tbody></table>');
}

/**
 * Render the raw report
 *
 * @param $report the report to render
 * @param $info true if information only, otherwise false.
 */
function render_raw_report($report, $info = false) {
  foreach ($report as $result) {
    print(strip_tags($result->title) .":\n");
    foreach ($result->status as $status) {
      print('  '. strip_tags($status) ."\n");
    }
    foreach ($result->warning as $warning) {
      print('  Warning: '. strip_tags($warning) ."\n");
    }
    foreach ($result->error as $error) {
      print('    Error: '. strip_tags($error) ."\n");
    }
  }
}

/**
 * Render the report.
 *
 * @param report the report to render
 */
function render_report($report) {
  print('<dl>');
  print('<dt><span class="title">'. t('PHP Environment Information') .'</span></dt>');
  print('<dd>');
  render_tabular_report($report['environment'], true);
  print('</dd>');

  print('<dt><span class="title">'. t('Drupal Requirements Tests') .'</span></dt>');
  print('<dd>');
  render_tabular_report($report['req_tests']);
  print('</dd>');

  print('<dt><span class="title">'. t('Alternative Report Formats') .'</span></dt>');
  print('<dd>');
  render_tabular_report($report['reports'], true);
  print('</dd>');
}

/**
 * Render the report as plain text.
 *
 * @param report the report to render
 */
function render_plaintext($report) {
  unset($report['environment']['Full PHP information page']); 
  unset($report['reports']);
  print(t('PHP Environment Information') ."\n");
  render_raw_report($report['environment'], true);
  print("\n");
  print(t('Drupal Requirements Tests') ."\n");
  render_raw_report($report['req_tests']);
}
